package dao

import (
	"context"
	"math/big"
	"pump/contract/bonding"
	"time"

	"github.com/ethereum/go-ethereum"
	"github.com/ethereum/go-ethereum/accounts/abi/bind"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/core/types"
)

func (d *Dao) GetBlockHeight(behindBlock ...int) (height int, err error) {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()
	n, err := d.ethClient.BlockNumber(ctx)
	if len(behindBlock) > 0 {
		n -= uint64(behindBlock[0])
		if n < 0 {
			n = 0
		}
	}
	return int(n), err
}

func (d *Dao) GetLogs(beginHeight, endHeight int, topics, addresses []string) (logs []types.Log, err error) {
	for i := 0; i < 2; i++ {
		// 重试2次
		logs, err = d.getLogs(beginHeight, endHeight, topics, addresses)
		if err == nil {
			return logs, nil
		}
	}
	return
}

func (d *Dao) getLogs(beginHeight, endHeight int, topics []string, addresses []string) (logs []types.Log, err error) {
	addrs := make([]common.Address, 0)
	for _, addr := range addresses {
		addrs = append(addrs, common.HexToAddress(addr))
	}

	q := ethereum.FilterQuery{
		FromBlock: big.NewInt(int64(beginHeight)),
		ToBlock:   big.NewInt(int64(endHeight)),
		Topics:    [][]common.Hash{{}},
		Addresses: addrs,
	}
	for _, topic := range topics {
		q.Topics[0] = append(q.Topics[0], common.HexToHash(topic))
	}

	ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
	defer cancel()
	return d.ethClient.FilterLogs(ctx, q)
}

func (d *Dao) GetTokenInfo(addr string) (info struct {
	Creator          common.Address
	Token            common.Address
	Pair             common.Address
	NftAddress       common.Address
	Data             bonding.BondingData
	Description      string
	Image            string
	Website          string
	Twitter          string
	Telegram         string
	Trading          bool
	TradingOnUniswap bool
	UniswapV2Pair    common.Address
}, err error) {
	ca, err := bonding.NewBonding(common.HexToAddress(d.c.Chain.BondingContract), d.ethClient)
	if err != nil {
		return
	}
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()

	opts := &bind.CallOpts{Context: ctx}

	return ca.TokenInfo(opts, common.HexToAddress(addr))
}
