package dbModel

import (
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
)

type Token struct {
	Id              int             `gorm:"primaryKey"`
	Creator         string          `gorm:"type:varchar(255);index;not null;comment:创建者"`
	Token           string          `gorm:"type:varchar(255);index;not null;comment:token"`
	NFTAddress      string          `gorm:"type:varchar(255);not null;comment:nft地址"`
	Pair            string          `gorm:"type:varchar(255);not null;comment:pair"`
	Ticker          string          `gorm:"type:varchar(255);not null;comment:token"`
	Name            string          `gorm:"type:varchar(255);not null;comment:名称"`
	Supply          decimal.Decimal `gorm:"type:decimal(65,0);comment:总量"`
	Description     string          `gorm:"type:varchar(10000);not null;comment:描述"`
	Image           string          `gorm:"type:varchar(255);not null;comment:图片"`
	Twitter         string          `gorm:"type:varchar(255);not null;comment:twitter"`
	Website         string          `gorm:"type:varchar(255);not null;comment:website"`
	Telegram        string          `gorm:"type:varchar(255);not null;comment:telegram"`
	UniswapV2Pair   string          `gorm:"type:varchar(255);not null;comment:uniswapV2Pair"`
	Graduated       bool            `gorm:"type:bool;not null;comment:内盘结束"`
	GraduatedTxHash string          `gorm:"type:varchar(255);not null;comment:内盘结束交易hash"`
	Price           decimal.Decimal `gorm:"type:decimal(40,18);comment:价格"`
	MarketCap       decimal.Decimal `gorm:"type:decimal(65,0);comment:市值"`
	Volume          decimal.Decimal `gorm:"type:decimal(65,0);comment:成交量"`
	Volume24H       decimal.Decimal `gorm:"type:decimal(65,0);column:volume24h;comment:24小时成交量"`
	Reserve0        decimal.Decimal `gorm:"type:decimal(65,0);comment:储备0"`
	Reserve1        decimal.Decimal `gorm:"type:decimal(65,0);comment:储备1"`
	TxHash          string          `gorm:"type:varchar(255);uniqueIndex;comment:tx hash"`
	BlockNumber     int             `gorm:"type:int;comment:区块高度"`
	BlockTime       int             `gorm:"type:int;index;comment:区块时间"`
	gorm.Model
}

func (*Token) TableName() string {
	return "public.token"
}

type TradeHistory struct {
	Id          int             `gorm:"primaryKey"`
	FromAddress string          `gorm:"type:varchar(255);index;not null;comment:用户地址"`
	Token       string          `gorm:"type:varchar(255);index;not null;comment:token"`
	AmountIn    decimal.Decimal `gorm:"type:decimal(40,18);comment:转入数量"`
	AmountOut   decimal.Decimal `gorm:"type:decimal(40,18);comment:转出数量"`
	Price       decimal.Decimal `gorm:"type:decimal(40,18);comment:价格"`
	IsBuy       bool            `gorm:"type:bool;not null;comment:是否是买入"`
	TxHash      string          `gorm:"type:varchar(255);not null;uniqueIndex:uidx_hash_idx;comment:tx hash"`
	LogIndex    int             `gorm:"type:int;not null;uniqueIndex:uidx_hash_idx;comment:log index"`
	BlockTime   int             `gorm:"type:int;index;comment:区块时间"`
	BlockNumber int             `gorm:"type:int;comment:区块高度"`
	gorm.Model
}

func (*TradeHistory) TableName() string {
	return "public.trade_history"
}

type Height struct {
	Key      string `gorm:"primaryKey"`
	IntValue int    `gorm:"type:int;not null"` // 配置value
}

func (*Height) TableName() string {
	return "public.height"
}
